/**
 * OpenGL 3.x with GLEW and GLUT 
 *
 * author	Po-Ying Li bylee@nchc.org.tw
 * version	1.0
 *
 * Homepage: http://viml.nchc.org.tw
 *
 * Copyright Po-Ying Li
 */
#ifndef _MYPROJECT
#define _MYPROJECT

float CalculateFov(float fFocus)
{
	float fRadtheta,fDegtheta;
	fRadtheta = 2.0 * atan2(24.0f/2.0f,fFocus);
	fDegtheta = (180.0 * fRadtheta) / PI;
	return fDegtheta;
}

float CalculateFov(float fSize, float fDistance)
{
	float fRadtheta,fDegtheta;
	fRadtheta = 2.0 * atan2(fSize / 2.0f, fDistance);
	fDegtheta = (180.0 * fRadtheta) / PI;
	return fDegtheta;
}

Matrix Perspective(float fFovY,float fAspect,float fNear,float fFar)
{
	Matrix mResult;
	float fRadians = fFovY / 2.0f * PI /180.0f;
	float fDeltaZ = fFar - fNear;
	float fSin = sin(fRadians);
	if((fDeltaZ == 0)||(fSin == 0) ||(fAspect == 0))
	{
		return mResult;
	}
	float fCot = cos(fRadians) / fSin;

	mResult.m44[0][0] = fCot / fAspect;
	mResult.m44[1][0] = 0;
	mResult.m44[2][0] = 0;
	mResult.m44[3][0] = 0;

	mResult.m44[0][1] = 0;
	mResult.m44[1][1] = fCot;
	mResult.m44[2][1] = 0;
	mResult.m44[3][1] = 0;

	mResult.m44[0][2] = 0;
	mResult.m44[1][2] = 0;
	mResult.m44[2][2] = -(fFar + fNear)/ fDeltaZ;
	mResult.m44[3][2] = -2.0 * fNear * fFar / fDeltaZ;

	mResult.m44[0][3] = 0;
	mResult.m44[1][3] = 0;
	mResult.m44[2][3] = -1;
	mResult.m44[3][3] = 0;

	return mResult;
}

Matrix Frustum(float fLeft,float fRight,float fBottom,float fTop,float fNear,float fFar)
{
	Matrix mResult;

	mResult.m44[0][0] = (2.0f*fNear)/(fRight-fLeft);
	mResult.m44[1][0] = 0;
	mResult.m44[2][0] = (fRight+fLeft)/(fRight-fLeft);
	mResult.m44[3][0] = 0;

	mResult.m44[0][1] = 0;
	mResult.m44[1][1] = (2.0f*fNear)/(fTop-fBottom);
	mResult.m44[2][1] = (fTop+fBottom)/(fTop-fBottom);
	mResult.m44[3][1] = 0;

	mResult.m44[0][2] = 0;
	mResult.m44[1][2] = 0;
	mResult.m44[2][2] = -(fFar+fNear)/(fFar-fNear);
	mResult.m44[3][2] = -(2.0f*fFar*fNear)/(fFar-fNear);

	mResult.m44[0][3] = 0;
	mResult.m44[1][3] = 0;
	mResult.m44[2][3] = -1;
	mResult.m44[3][3] = 0;
	return mResult;
}

Matrix Ortho(float fLeft,float fRight,float fBottom,float fTop,float fNear,float fFar)
{
	Matrix mResult;

	mResult.m44[0][0] = 2.0f/(fRight-fLeft);
	mResult.m44[1][0] = 0;
	mResult.m44[2][0] = 0;
	mResult.m44[3][0] = -(fRight+fLeft)/(fRight-fLeft);

	mResult.m44[0][1] = 0;
	mResult.m44[1][1] = 2.0f/(fTop-fBottom);
	mResult.m44[2][1] = 0;
	mResult.m44[3][1] = -(fTop+fBottom)/(fTop-fBottom);

	mResult.m44[0][2] = 0;
	mResult.m44[1][2] = 0;
	mResult.m44[2][2] = -2.0f/(fFar-fNear);
	mResult.m44[3][2] = -(fFar+fNear)/(fFar-fNear);

	mResult.m44[0][3] = 0;
	mResult.m44[1][3] = 0;
	mResult.m44[2][3] = 0;
	mResult.m44[3][3] = 1;
	return mResult;
}

#endif